﻿using System;

namespace ConfigurationHelper.Classes
{
    /// <summary>
    /// Container for application settings which currently focuses on databases settings
    /// </summary>
    public class ApplicationSettings
    {
        private static readonly Lazy<ApplicationSettings> Lazy = new(() => new ApplicationSettings());
        
        /// <summary>
        /// Access point to methods and properties
        /// </summary>
        public static ApplicationSettings Instance => Lazy.Value;
       
        /// <summary>
        /// Indicates whether EF Core logging is enabled
        /// </summary>
        public bool UsingLogging { get; set; }

        /// <summary>
        /// Type of logging <see cref="LoggingDestination"/>
        /// </summary>
        public LoggingDestination LoggingDestination { get; set; }

        /// <summary>
        /// File name when <see cref="Classes.LoggingDestination"/> is
        /// set to <see cref="Classes.LoggingDestination.LogFile"/>
        /// </summary>
        public string LogFileName { get; set; }

        /// <summary>
        /// Using production or development (refactor for more environments)
        /// </summary>
        public bool IsProduction { get; set; }

        /// <summary>
        /// Database connection string
        /// </summary>
        public string ConnectionString { get; set; }

        /// <summary>
        /// Get settings from appsettings.json
        /// </summary>
        private ApplicationSettings()
        {
            UsingLogging = Helper.GetLogOptions().Use;
            LoggingDestination = Helper.GetLogOptions().Destination;
            LogFileName = Helper.GetLogOptions().LogFileName;
            IsProduction = Helper.GetEnvironment().IsProduction;
            ConnectionString = Helper.GetConnectionString();
        }
        
    }

}
