﻿using System;
using System.Collections.Generic;
using System.IO;
using ConfigurationHelper.Classes;
using Microsoft.Extensions.Configuration;
using Environment = ConfigurationHelper.Classes.Environment;

namespace ConfigurationHelper
{
    /// <summary>
    /// Provides service to read appsettings.json
    /// </summary>
    public class Helper
    {
        
        /// <summary>
        /// Configuration file name to read from.
        /// </summary>
        public static string ConfigurationFileName { get; set; } = "appsettings.json";
      
        /// <summary>
        /// Get connection string based on environment
        /// </summary>
        public static string GetConnectionString()
        {
            return ConfigurationMainBuilderRoot()
                .GetConnectionString(InitOptions<Environment>("Environment").IsProduction ?
                    "ProductionConnection" :
                    "DevelopmentConnection");
        }


        /// <summary>
        /// Determines if application is running dev or prod environment
        /// </summary>
        /// <returns></returns>
        public static Environment GetEnvironment()
        {
            InitializeConfiguration();
            return InitOptions<Environment>("Environment");
        }

        /// <summary>
        /// Read log settings/options
        /// </summary>
        /// <returns><see cref="Logging"/> object</returns>
        public static Logging GetLogOptions()
        {
            InitializeConfiguration();
            return InitOptions<Logging>("LogOptions");
        }

        /// <summary>
        /// Initialize ConfigurationBuilder for appsettings
        /// </summary>
        /// <returns>IConfigurationRoot</returns>
        private static IConfigurationRoot InitializeConfiguration()
        {

            var builder = new ConfigurationBuilder()
                .SetBasePath(Directory.GetCurrentDirectory())
                .AddJsonFile(ConfigurationFileName);

            return builder.Build();

        }
        /// <summary>
        /// Initialization 
        /// </summary>
        /// <returns><see cref="IConfigurationRoot"/></returns>
        private static IConfigurationRoot ConfigurationMainBuilderRoot()
        {
            var builder = new ConfigurationBuilder();
            builder.AddJsonFile(ConfigurationFileName, optional: false);

            var configuration = builder.Build();
            return configuration;
            
        }
        /// <summary>
        /// Generic method to read a section from the json configuration file.
        /// </summary>
        /// <typeparam name="T">Class type</typeparam>
        /// <param name="section">Section to read</param>
        /// <returns>Instance of T</returns>
        public static T InitOptions<T>(string section) where T : new()
        {
            var config = InitializeConfiguration();
            return config.GetSection(section).Get<T>();
        }
    }
}
