﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace UnitTestsProject.Base
{
    /// <summary>
    /// Traits for unit test methods displayed in test explorer
    /// </summary>
    public enum Trait
    {
        /// <summary>
        /// Place holder 
        /// </summary>
        PlaceHolder,
        /// <summary>
        /// For testing UnixEpochDateTime operations
        /// </summary>
        UnixEpochDateTime,
        /// <summary>
        /// For testing configuration helper methods
        /// </summary>
        Appsettings,
        /// <summary>
        /// For testing date time language extensions
        /// </summary>
        DateTimeExtension
    }
    /// <summary>
    /// Declarative class for using Trait enum about for
    /// traits on test method.
    /// </summary>
    public class TestTraitsAttribute : TestCategoryBaseAttribute
    {
        private readonly Trait[] _traits;

        public TestTraitsAttribute(params Trait[] traits)
        {
            _traits = traits;
        }

        public override IList<string> TestCategories => 
            _traits.Select(trait => 
                Enum.GetName(typeof(Trait), trait)).ToList();
    }

}