﻿using ConfigurationHelper.Classes;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using UnitTestsProject.Base;
using static ConfigurationHelper.Classes.ApplicationSettings;

namespace UnitTestsProject
{
    /// <summary>
    /// Test methods for <see cref="ApplicationSettings"/>
    /// </summary>
    [TestClass]
    public partial class ConfigurationTest : TestBase
    {
        #region AppSettings test (uses static using statement)

        [TestMethod]
        [TestTraits(Trait.Appsettings)]
        public void AppSettings_ReadConnectionString()
        {
            var expected =
                "Server=.\\SQLEXPRESS;" +
                "Database=NorthWind2020;" +
                "Integrated Security=true";

            string ConnectionString = Instance.ConnectionString;

            Assert.AreEqual(expected, ConnectionString);

        }

        /// <summary>
        /// Validate connection string can be read
        /// </summary>
        [TestMethod]
        [TestTraits(Trait.Appsettings)]
        public void SqlServerEnvironment()
        {
            Assert.IsFalse(Instance.IsProduction);
        }

        /// <summary>
        /// Validate reading UsingLogging for
        /// Entity Framework Core
        /// </summary>
        [TestMethod]
        [TestTraits(Trait.Appsettings)]
        public void UseLogging()
        {
            Assert.IsTrue(Instance.UsingLogging);
        }

        /// <summary>
        /// Validate logging destination for
        /// Entity Framework Core
        /// </summary>
        [TestMethod]
        [TestTraits(Trait.Appsettings)]
        public void Log_Destination()
        {
            Assert.IsTrue(
                Instance.LoggingDestination ==
                LoggingDestination.LogFile);
        }

        #endregion

    }

}