using System;
using System.Diagnostics;
using System.Threading.Tasks;
using SystemTextJsonLibrary.Containers;
using SystemTextJsonLibrary.Converters;
using SystemTextJsonLibrary.LanguageExtensions;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using UnitTestsProject.Base;
using static SystemTextJsonLibrary.Operations;


namespace UnitTestsProject
{
    /// <summary>
    /// Test methods for SystemTextJsonLibrary
    /// </summary>
    [TestClass]
    public partial class JsonTest : TestBase
    {
        /// <summary>
        /// Validate returning date information using
        /// <seealso cref="UnixEpochLocalDateTimeConverter"/>
        /// converter from Microsoft
        /// </summary>
        /// <returns>N/A</returns>
        [TestMethod]
        [TestTraits(Trait.UnixEpochDateTime)]
        public async Task Deserialize_Object_With_UpTime()
        {
            DateContainer dateContainer = 
                await GetPartialComputerInformationTask(
                    _computerJsonFileName);
            
            var up = GetUpTime();

            var timeSpan1 = new TimeSpan(
                dateContainer.OsUptime.Days, 
                dateContainer.OsUptime.Hours, 
                dateContainer.OsUptime.Minutes, 
                0);
            
            var timeSpan2 = new TimeSpan(
                up.Days, 
                up.Hours, 
                up.Minutes, 
                0);

            Assert.AreEqual(timeSpan1, timeSpan2);
            
        }

        /// <summary>
        /// No assertion as the conversion fails on OsLocalDateTime
        /// compared to current time in hours and AM/PM indicator
        /// </summary>
        /// <returns>N/A</returns>
        [TestMethod]
        [TestTraits(Trait.UnixEpochDateTime)]
        public async Task Deserialize_Object_With_UnixEpochDateTime()
        {
            DateContainer dateContainer =
                await GetPartialComputerInformationTask(
                    _computerJsonFileName);

            var compareResult = DateTime.Compare(
                dateContainer.OsLocalDateTime
                    .RemoveMillisecondsAndSeconds(),
                DateTime.Now.RemoveMillisecondsAndSeconds());

            Assert.AreEqual(compareResult, 0,
                "Expected UnixEpochDateTime to equal DateTime.Now");

        }

        #region DateTime language extensions

        [TestMethod]
        [TestTraits(Trait.DateTimeExtension)]
        public void RemoveMilliseconds()
        {
            var dateTime = DateTime.Today.RemoveMilliseconds();
            Assert.IsTrue(dateTime.Millisecond == 0);
        }

        [TestTraits(Trait.DateTimeExtension)]
        [TestMethod]
        public void RemoveSeconds()
        {
            var dateTime = DateTime.Today.RemoveSeconds();
            Assert.IsTrue(dateTime.Second == 0 && dateTime.Millisecond == 0);
        }

        [TestTraits(Trait.DateTimeExtension)]
        [TestMethod]
        public void RemoveMillisecondsAndSeconds()
        {
            var dateTime = DateTime.Today.RemoveMillisecondsAndSeconds();
            Assert.IsTrue(dateTime.Second == 0 && dateTime.Millisecond == 0);
        }

        #endregion
    }
}
