﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;

namespace Publisher.Classes
{
    public static class DirectoryHelper
    {
        public static string RemoveWhitespace(this string input) => 
            new(input.ToCharArray().Where(c => !char.IsWhiteSpace(c)).ToArray());

        /// <summary>
        /// Given a folder name return all parents according to level
        /// </summary>
        /// <param name="folderName">Sub-folder name</param>
        /// <param name="level">Level to move up the folder chain</param>
        /// <returns>List of folders dependent on level parameter</returns>
        public static string UpperFolder(this string folderName, int level)
        {
            var folderList = new List<string>();

            while (!string.IsNullOrWhiteSpace(folderName))
            {
                var parentFolder = Directory.GetParent(folderName);
                if (parentFolder == null)
                {
                    break;
                }

                folderName = Directory.GetParent(folderName)?.FullName;
                folderList.Add(folderName);
            }

            if (folderList.Count > 0 && level > 0)
            {
                if (level - 1 <= folderList.Count - 1)
                {
                    return folderList[level - 1];
                }
                else
                {
                    return folderName;
                }
            }
            else
            {
                return folderName;
            }
        }
        /// <summary>
        /// Given a path for a project obtains it's solution folder
        /// </summary>
        /// <returns>Current solution folder from project folder</returns>
        public static string CurrentSolutionFolder() => AppDomain.CurrentDomain.BaseDirectory.UpperFolder(5);
        public static string SolutionName() => Directory.GetFiles(CurrentSolutionFolder(), "*.sln").FirstOrDefault();
        /// <summary>
        /// 
        /// </summary>
        /// <param name="folder"></param>
        /// <param name="extension"></param>
        /// <returns></returns>
        public static string ProjectName(string folder, string extension = "csproj") => 
            Directory.GetFiles(folder, $"*.{extension}").FirstOrDefault();

        public static List<ProjectItem> ProjectItems(List<string> projectNamesList)
        {
            List<ProjectItem> items = new List<ProjectItem>();

            foreach (var name in projectNamesList)
            {
                var path = Path.Combine(CurrentSolutionFolder(), name, "Bin", "Debug");

                if (Directory.Exists(path))
                {
                    var packNames = Directory.GetFiles(path, "*.nupkg");
                    if (packNames.Length > 0)
                    {

                        var item = new ProjectItem {Name = name};
                        
                        foreach (var packName in packNames)
                        {
                            var version = Path.GetFileNameWithoutExtension(packName);
                            version = version.Substring( version.IndexOf(".", StringComparison.Ordinal) +1);
          
                            item.PackageList.Add(new Package()
                            {
                                Name = Path.GetFileName(packName), 
                                Version = new Version(version)
                            });
                            
                            item.Path = Path.GetDirectoryName(packName);
                        }

                        items.Add(item);
                    }
                }
            }

            return items;

        }
    }
}