﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Publisher.Classes
{
    public class PackHelpers
    {

        public static string NuGetExecutable => ApplicationSettings.Instance.NuGetExecutable;
        public static string PackageLocation => ApplicationSettings.Instance.PackageLocation;

        /// <summary>
        /// Install package
        /// </summary>
        /// <param name="package">Desired package</param>
        /// <param name="showCommand">When true command is copied to the Windows Clipboard</param>
        /// <returns>N/A</returns>
        public static async Task Publish(string package, bool showCommand = false)
        {
            if (showCommand)
            {
                Clipboard.SetText($"{NuGetExecutable} add {package} -source {PackageLocation}");
            }
            
            var start = new ProcessStartInfo
            {
                FileName = NuGetExecutable,
                RedirectStandardOutput = true,
                Arguments = $"add {package} -source {PackageLocation}",
                CreateNoWindow = true
            };


            using var process = Process.Start(start);
            await process.WaitForExitAsync();

        }

        /// <summary>
        /// Open local feed folder
        /// </summary>
        public static void OpenFeedFolder()
        {
            var start = new ProcessStartInfo
            {
                FileName = PackageLocation,
                UseShellExecute = true
            };
            
            Process.Start(start);
            
        }

    }
}
