﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text.Json;
using SystemTextJsonLibrary.Converters;

namespace SystemTextJsonLibrary
{
	/// <summary>
	/// Serialize and deserialize methods for json
	/// </summary>
	public class JSonHelper
	{
		/// <summary>
		/// Deserialize from Json string to T
		/// </summary>
		/// <typeparam name="T">Type to deserialize Json to</typeparam>
		/// <param name="json">Valid json for deserialize T too.</param>
		/// <returns>single instance of T</returns>
		public static T DeserializeObject<T>(string json) => JsonSerializer.Deserialize<T>(json);
		/// <summary>
		/// Deserialize from Json string to T using <see cref="UnixEpochLocalDateTimeConverter"/>
		/// </summary>
		/// <typeparam name="T">Type to deserialize Json to</typeparam>
		/// <param name="json">Valid json for deserialize T too.</param>
		/// <returns>single instance of T</returns>
		public static T DeserializeObjectUnixEpochDateTime<T>(string json)
		{

			JsonSerializerOptions options = new();
			options.Converters.Add(new UnixEpochLocalDateTimeConverter());

			return JsonSerializer.Deserialize<T>(json, options);

		}

		/// <summary>
		/// Deserialize from Json string to List(Of T)
		/// </summary>
		/// <typeparam name="T">Type to deserialize Json to</typeparam>
		/// <param name="json">Valid json for deserialize T too</param>
		/// <returns>List(Of T)</returns>
		public static List<T> JSonToList<T>(string json)
		{

			JsonSerializerOptions options = new();
			options.Converters.Add(new UnixEpochLocalDateTimeConverter());

			return JsonSerializer.Deserialize<List<T>>(json);

		}

		/// <summary>
		/// Serialize a list to a file
		/// </summary>
		/// <typeparam name="T"></typeparam>
		/// <param name="sender">Type to serialize</param>
		/// <param name="fileName">File json to this file</param>
		/// <param name="format">true to format json, otherwise no formatting</param>
		/// <returns>Value Tuple, on success return true/null, otherwise false and the exception thrown</returns>
		public static (bool result, Exception exception) JsonToListFormatted<T>(List<T> sender, string fileName, bool format = true)
		{

			try
			{

				var options = new JsonSerializerOptions { WriteIndented = true };
				File.WriteAllText(fileName, JsonSerializer.Serialize(sender, (format ? options : null)));

				return (true, null);

			}
			catch (Exception e)
			{
				return (false, e);
			}

		}

	}
}
