﻿using System.Diagnostics;
using System.IO;
using System.Threading.Tasks;
using SystemTextJsonLibrary.Containers;

namespace SystemTextJsonLibrary
{
    /// <summary>
    /// PowerShell operations for obtaining computer information
    /// </summary>
    public class Operations
    {
        /// <summary>
        /// Get the following information
        ///
        /// - Bios Release Date
        /// - Os Local Date Time
        /// - Os Last Boot Up Time
        /// - Os Up time
        ///
        /// By invoking a PowerShell command which produces a json file then
        /// deserializes to concrete containers.
        /// </summary>
        /// <param name="fileName"></param>
        /// <returns><see cref="DateContainer"/> container</returns>
        public static async Task<DateContainer> GetPartialComputerInformationTask(string fileName)
        {

            var start = new ProcessStartInfo
            {
                FileName = "powershell.exe",
                RedirectStandardOutput = true,
                Arguments = "Get-ComputerInfo | select BiosReleaseDate,OsLocalDateTime,OsLastBootUpTime,OsUptime | ConvertTo-Json",
                CreateNoWindow = true
            };

            using var process = Process.Start(start);
            using var reader = process.StandardOutput;

            process.EnableRaisingEvents = true;

            var fileContents = await reader.ReadToEndAsync();

            await File.WriteAllTextAsync(fileName, fileContents);
            await process.WaitForExitAsync();

            // read back json created above via PowerShell using Process.Start
            var json = await File.ReadAllTextAsync(fileName);

            return JSonHelper.DeserializeObjectUnixEpochDateTime<DateContainer>(json);

        }
    }
}